<?php

/* Make vim work like it should */
/* vim: set foldmethod=marker expandtab tabstop=4 shiftwidth=4: */

/**
* My Statistics Library - returns various MySQL stats for the passed DB handle
*
* @package MyStats
* @author  Mark Cotner
* @filesource
* $Id$
*/

require_once('config.php');

/**
* MyStats class
* <br>NOTE: INTENDED TO BE CALLED STATICALLY<br>
*
* @return       boolean
* @package      MyStats
* @author       Mark Cotner
*/

class MyStats
{

    /**
    * constructor
    *
    * @return   boolean     true on success, false on error
    */
    function MyStats() {
        return true;
    }

    /**
    * getStatus - get all server status variables
    *
    * @param    object      $db handle for database to pull stats
    * @return   array       assoc array containing status => value pairs
    */
    function getStatus($db) {
        $res = $db->getAll('SHOW STATUS');
        foreach ($res as $row) {
            $output[$row[0]] = $row[1];
        }
        return $output;
    }

    /**
    * getQueryCacheHitRate
    *
    * @param    object      $db handle for database to pull stats
    * @return   float       Query Cache Hit Rate Percentage to the thousandth
    */
    function getQueryCacheHitRate($db) {
        $status = MyStats::getStatus($db);
        if ($status['Qcache_hits'] > 0) {
            return round(100 - ($status['Com_select'] / $status['Qcache_hits']) * 100, 2);
        } else {
            return 0;
        }
    }

    /**
    * getServerVersion
    *
    * @param    object      $db handle
    * @return   string      Server Version Information
    */
    function getServerVersion($db) {
        return $db->getOne('SELECT version()');
    }

    /**
    * getServerUptime
    *
    * @param    object      $db handle
    * @return   string      Server Uptime
    */
    function getServerUptime($db) {
        $status = MyStats::getStatus($db);
        if ($status['Uptime'] > 86400 and $status['Uptime'] < 86400 * 2) {
            return sprintf('%s day, %02d:%02d:%02d',
                round($status['Uptime'] / 86400),
                round(($status['Uptime'] % 86400) / 3600),
                round((($status['Uptime'] % 86400) % 3600) / 60),
                ((($status['Uptime'] % 86400 ) % 3600) % 60));
        } else if ($status['Uptime'] > 86400) {
            return sprintf('%s days, %02d:%02d:%02d',
                round($status['Uptime'] / 86400),
                round(($status['Uptime'] % 86400) / 3600),
                round((($status['Uptime'] % 86400) % 3600) / 60),
                ((($status['Uptime'] % 86400 ) % 3600) % 60));
        } else {
            return sprintf('%02d:%02d:%02d',
                round($status['Uptime'] / 3600),
                round(($status['Uptime'] % 3600) / 60),
                (($status['Uptime'] % 3600) % 60));
        }
    }

    /**
    * getServerUptimeSeconds
    *
    * @param    object      $db handle
    * @return   string      Server Uptime
    */
    function getServerUptimeSeconds($db) {
        $status = MyStats::getStatus($db);
        return $status['Uptime'];
    }

    /**
    * getTotalQueries
    *
    * @param    object      $db handle for database FROM which to pull stats
    * @return   int         Total DB Queries
    */
    function getTotalQueries($db) {
        $status = MyStats::getStatus($db);
        return $status['Questions'];
    }

    /**
    * getSlowQueries
    *
    * @param    object      $db handle for database FROM which to pull stats
    * @return   int         Slow DB Queries
    */
    function getSlowQueries($db) {
        $status = MyStats::getStatus($db);
        return $status['Slow_queries'];
    }

    /**
    * getTableScans
    *
    * @param    object      $db handle for database FROM which to pull stats
    * @return   int         Table Scans
    */
    function getTableScans($db) {
        $status = MyStats::getStatus($db);
        return $status['Select_scan'];
    }

    /**
    * getTableLocksWaited
    *
    * @param    object      $db handle for database FROM which to pull stats
    * @return   float       Percentage of table locks that had to wait
    */
    function getTableLocksWaited($db) {
        $status = MyStats::getStatus($db);
        return round($status['Table_locks_waited'] / ($status['Table_locks_immediate'] + $status['Table_locks_waited']) * 100, 2);
    }

    /**
    * getIndexHitRate {
    *
    * @param    object      $db handle for database FROM which to pull stats
    * @return   float       Index Hit Rate Percentage to the thousandth
    */
    function getIndexHitRate($db) {
        $status = MyStats::getStatus($db);
        return round(100 - ($status['Key_reads'] / $status['Key_read_requests']) * 100, 2);
    }

    /**
    * addQueryHistory
    *
    * @param    object      $db     handle of pgtop DB
    * @param    string      $dbase  database query was run against
    * @param    string      $query  query to add to history
    * @return   boolean     true on success, false on failure
    */
    function addQueryHistory($db, $dbase, $query) {
        if ($db->query("INSERT INTO query_history (dbase, query) VALUES ('$dbase', '$query')")) {
            return true;
        } else {
            return false;
        }
    }

    /**
    * getQueryHistory
    *
    * @param    object      $db handle for pgtop DB
    * @return   mixed       struct containing queries and frequency
    */
    function getQueryHistory($db) {
        $db->setFetchMode(DB_FETCHMODE_ASSOC);
        $result = $db->query(   "SELECT
                                  query,
                                  max(dbase) AS dbase,
                                  max(ts) AS last_noticed,
                                  count(query) AS count 
                                 FROM 
                                  query_history 
                                 GROUP BY 
                                  query
                                 ORDER BY
                                  count DESC");
        while($row = $result->fetchRow()) {
            $output[] = $row;
        }
        $db->setFetchMode(DB_FETCHMODE_ORDERED);
        return $output;
    }

    /**
    * clearQueryHistory
    *
    * @param    object      $db handle for pgtop DB
    * @return   boolean     true on success, false on error
    */
    function clearQueryHistory($db) {
        if ($db->query("TRUNCATE query_history")) {
            return true;
        } else {
            return false;
        }
    }

    /**
    * getExplainQuery
    *
    * @param    object      $db handle for database FROM which to run explain query
    * @param    string      Query to run explain against
    * @return   string      Explain Output
    */
    function getExplainQuery($db, $query) {
        $result = $db->getAll("EXPLAIN $query");
        $output = '';
        foreach ($result as $idx) {
            $output .= $idx[0]."\n";
        }
        return $output;
    }

    /**
    * getExplainAnalyzeQuery
    *
    * @param    object      $db handle for database FROM which to run explain analyze query
    * @param    string      Query to run explain analyze against
    * @return   string      Explain Analyze Output
    */
    function getExplainAnalyzeQuery($db, $query) {
        $result = $db->getAll("EXPLAIN ANALYZE $query");
        $output = '';
        foreach ($result as $idx) {
            $output .= $idx[0]."\n";
        }
        return $output;
    }

    /**
    * listProcesses
    *
    * @param    object      $db handle for database FROM which to get queries
    * @return   mixed       struct containing queries and statistics
    */
    function listProcesses($db) {
        $db->setFetchMode(DB_FETCHMODE_ASSOC);
        $result = $db->query("SHOW FULL PROCESSLIST");
        while($row = $result->fetchRow()) {
            $output[] = $row;
        }
        $db->setFetchMode(DB_FETCHMODE_ORDERED);
        return $output;
    }

    /**
    * listProcessesNoIdle
    *
    * @param    object      $db handle for database FROM which to get queries
    * @return   mixed       struct containing queries and statistics
    */
    function listProcessesNoIdle($db) {
        $db->setFetchMode(DB_FETCHMODE_ASSOC);
        $result = $db->query("SHOW FULL PROCESSLIST");
        while($row = $result->fetchRow()) {
            if ($row['Command'] != 'Sleep') {
                $output[] = $row;
            }
        }
        $db->setFetchMode(DB_FETCHMODE_ORDERED);
        return $output;
    }

    /**
    * listDbs
    *
    * @param    object      $db handle for connection
    * @return   array       numerical array of dbs
    */
    function listDbs($db) {
        $result = $db->query('SHOW DATABASES');
        while ($row = $result->fetchRow()) {
            $output[] = $row[0];
        }
        return $output;
    }

    /**
    * listTableStats
    *
    * @param    object      $db     handle for db
    * @return   array       array containing tables in db
    */
    function listTableStats($db) {
        $db->setFetchMode(DB_FETCHMODE_ASSOC);
        $result = $db->query('SHOW TABLE STATUS');
        if (DB::isError($db)) {
                die($db->getMessage());
        }
        while($row = $result->fetchRow()) {
            $output[] = $row;
        }
        $db->setFetchMode(DB_FETCHMODE_ORDERED);
        return $output;
    }

    /**
    * getThreadsTotal
    *
    * @param    object      $db     handle for db
    * @return   integer     total number of threads
    */
    function getThreadsTotal($db) {
        $status = MyStats::getStatus($db);
        return $status['Threads_connected'];
    }

    /**
    * getThreadsActive
    * @param    object      $db     handle for db
    * @return   integer     number of active threads
    */
    function getThreadsActive($db) {
        $status = MyStats::getStatus($db);
        return $status['Threads_running'];
    }

    /**
    * getThreadsIdle
    *
    * @param    object      $db     handle for db
    * @return   integer     number of active threads
    */
    function getThreadsIdle($db) {
        $status = MyStats::getStatus($db);
        return $status['Threads_connected'] - $status['Threads_running'];
    }

    /**
    * getNumSelects
    *
    * @param    object      $db     handle for db
    * @return   integer     number of select queries
    */
    function getNumSelects($db) {
        $status = MyStats::getStatus($db);
        return $status['Com_select'];
    }

    /**
    * getNumInserts
    *
    * @param    object      $db     handle for db
    * @return   integer     number of insert queries
    */
    function getNumInserts($db) {
        $status = MyStats::getStatus($db);
        return $status['Com_insert'] + $status['Com_insert_select'];
    }

    /**
    * getNumUpdates
    *
    * @param    object      $db     handle for db
    * @return   integer     number of update queries
    */
    function getNumUpdates($db) {
        $status = MyStats::getStatus($db);
        return $status['Com_update'] + $status['Com_update_multi'];
    }

    /**
    * getNumDeletes
    *
    * @param    object      $db     handle for db
    * @return   integer     number of delete queries
    */
    function getNumDeletes($db) {
        $status = MyStats::getStatus($db);
        return $status['Com_delete'] + $status['Com_delete_multi'];
    }
}

/*
// Testing
my_print_r(MyStats::getCacheHitRate($GLOBALS['dbconn'][0]));
my_print_r(MyStats::getTotalQueries($GLOBALS['dbconn'][0]));
my_print_r(MyStats::getIndexHitRate($GLOBALS['dbconn'][0]));
my_print_r(MyStats::getIndexCacheHitRate($GLOBALS['dbconn'][0]));
my_print_r(MyStats::getExplainQuery($GLOBALS['dbconn'][0], 'SELECT * FROM pg_stat_activity'));
my_print_r(MyStats::getExplainAnalyzeQuery($GLOBALS['dbconn'][0], 'SELECT * FROM pg_stat_activity'));
my_print_r(MyStats::listProcesses($GLOBALS['dbconn'][0]));
#my_print_r(MyStats::runVacuumDb($GLOBALS['dbconn'][0]));
#my_print_r(MyStats::runAnalyzeDb($GLOBALS['dbconn'][0]));
my_print_r(MyStats::runVacuumTable($GLOBALS['dbconn'][0], 'mark'));
my_print_r(MyStats::runAnalyzeTable($GLOBALS['dbconn'][0], 'mark'));
my_print_r(MyStats::listTableStats($GLOBALS['dbconn'][0]));
my_print_r(MyStats::getQueryHistory($GLOBALS['pgtop_dbconn']));
my_print_r(MyStats::listDbs($GLOBALS['dbconn'][0]));
my_print_r(MyStats::getStatus($GLOBALS['dbconn'][0]));
my_print_r(MyStats::getQueryCacheHitRate($GLOBALS['dbconn'][0]));
my_print_r(MyStats::getServerUptime($GLOBALS['dbconn'][0]));
my_print_r(MyStats::listProcesses($GLOBALS['dbconn'][0]));
my_print_r(MyStats::listDbs($GLOBALS['dbconn'][0]));
my_print_r(MyStats::listTableStats($GLOBALS['dbconn'][0]));
*/


?>
